# Copyright (c) 2006 by BlueDynamics Tyrol - Klein und Partner KEG, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Jens Klein <jens@bluedynamics.com>"""
__docformat__ = 'plaintext'

import socket, os
from Acquisition import Implicit, aq_parent, aq_base, aq_inner
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Globals import package_home
from BTrees.OOBTree import OOBTree
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.UserPropertySheet import UserPropertySheet
from Products.PluggableAuthService.interfaces.plugins import ICredentialsUpdatePlugin
from Products.PluggableAuthService.interfaces.plugins import IPropertiesPlugin
from Products.CMFCore.utils import getToolByName

_wwwdir = os.path.join( package_home( globals() ), 'www' )

manage_addStoreAndRecallPasswordMultiPluginForm = PageTemplateFile(
    os.path.join(_wwwdir, 'addStoreAndRecallPasswordMultiPlugin'), 
    globals(),
    __name__='manage_addStoreAndRecallPasswordMultiPlugin')

def addStoreAndRecallPasswordMultiPlugin(self, id, title='', REQUEST=None):
    """Factory method to instantiate a StoreAndRecallPasswordMultiPlugin."""
    
    # Make sure we really are working in our container (the 
    # PluggableAuthService object)
    self = self.this()

    # Instantiate the folderish adapter object
    plugin = StoreAndRecallPasswordMultiPlugin(id, title=title)
    self._setObject(id, plugin)

    if REQUEST is not None:
        REQUEST.RESPONSE.redirect('%s/manage_main' % self.absolute_url())
        
class StoreAndRecallPasswordMultiPlugin(BasePlugin):
    ''' fetches group info from LDAP and provides in PAS'''

    security = ClassSecurityInfo()
    __implements__ = (getattr(BasePlugin,'__implements__',()),)
    meta_type = 'StoreAndRecallPasswordMultiPlugin'
        
    def __init__(self, id, title=None):
        self.id = id
        self.title = title
        self.usersandpasswords = OOBTree()
        
    #---------------------------------------------------------------------------
    # (Plone)PAS interface defined methods
        
    # implements interface IPropertiesPlugin (PAS)    
    security.declarePrivate('getPropertiesForUser')
    def getPropertiesForUser(self, user, request=None ):
        """ user -> {}

        o user will implement IPropertiedUser.
        """
        try:
            password = self.usersandpasswords[user.getUserName()]
        except KeyError:
            return None
        
        ps = UserPropertySheet(self.id, password=password)
        # here we need a subclass of the UserPropertySheet to ensure security.
        return ps

    security.declarePrivate('updateCredentials')
    def updateCredentials(self, request, response, login, new_password):
        """ Respond to change of credentials. """
        # get user for login: XXX
        userid = login
        self.usersandpasswords[userid] = new_password
    
                    
classImplements(StoreAndRecallPasswordMultiPlugin, 
                IPropertiesPlugin,
                ICredentialsUpdatePlugin,
)
InitializeClass(StoreAndRecallPasswordMultiPlugin)
